package kr.pe.nowonbun.filetransfer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;
import java.util.Stack;
public class FileTransferClient extends Socket implements Runnable{

	private Thread thread = null;
	private InputStream receiver = null;
	private OutputStream sender = null;
	private Stack<Exception> errorList = null;
	private FileTransferAddress address = null;
	private File savefile = null;
	private FileTransferListener listener = null;
	protected FileTransferClient(File savefile,int processSize) throws IOException{
		this.savefile = savefile;
	}
	protected void serverStart() throws IOException{
		thread = new Thread(this);
		receiver = getInputStream();
		sender = getOutputStream();
		errorList = new Stack<Exception>();
		thread.start();
	}
	public FileTransferClient(FileTransferAddress address,File savefile) throws IOException{
		this.savefile = savefile;
		this.address = address;
		thread = new Thread(this);
		errorList = new Stack<Exception>();
	}
	private void connection() throws IOException{
		super.connect(address);
		receiver = getInputStream();
		sender = getOutputStream();
		thread.start();
	}
	@Override
	public void close() throws IOException{
		super.close();
		if(listener != null){
			listener.connectionClose();
		}
	}
	public void setFileTransferListener(FileTransferListener listener){
		this.listener = listener;
	}
	@Override
	public void run() {
		byte[] lengthData = null;
		int length = 0;
		String filename = "";
		FileOutputStream out = null;
		try{
			lengthData = new byte[FileTransferBitConverter.INTBITSIZE];
			receiver.read(lengthData,0,lengthData.length);
			length = FileTransferBitConverter.toInt32(lengthData, 0);
			if(length == 0){
				return;
			}
			if(listener != null){
				listener.downloadStart();
			}
			byte[] filenamebyte = new byte[length];
			receiver.read(filenamebyte,0,filenamebyte.length);
			filename = new String(filenamebyte);
			File file = new File(savefile.getPath() + "\\" + filename);
			if(file.exists()) file.delete();

			out = new FileOutputStream(file);
			receiver.read(lengthData,0,lengthData.length);
			length = FileTransferBitConverter.toInt32(lengthData, 0);
			if(length == 0){
				return;
			}
			receiveWrite(out,length,listener);

			if(listener != null){
				listener.downloadComplate();
				listener.fileSaveComplate(savefile.getPath() + "\\" + filename);
			}
		} catch (Exception e) {
			if(listener != null){
				listener.receiveError(e);
			}
			errorList.push(e);

		} finally{
			try{
				if(isConnected()){
					close();
				}
				if(out != null){
					out.close();
				}
			}catch(Exception ex){
				if(listener != null){
					listener.receiveError(ex);
				}
				errorList.push(ex);
			}
		}
	}
	public FileTransferAddress getAddress(){
		return this.address;
	}
	public void sendFile(File file) throws FileTransferException,IOException{
		connection();
		if(file == null) throw new FileTransferException("File path not setting");
		if(!file.isFile()) throw new FileTransferException("File path not setting");
		if(!isConnected()) throw new FileTransferException("Socket is closed");
		String filename = file.getName();
		if(filename == null) throw new FileTransferException("File path not setting");
		FileInputStream in = null;
		byte[] databyte = null;
		byte[] filenamebyte = filename.getBytes();
		try{
			if(listener != null){
				listener.uploadStart();
			}
			in = new FileInputStream(file);
			byte[] length = FileTransferBitConverter.getBytes(filenamebyte.length);
			sender.write(length,0,FileTransferBitConverter.INTBITSIZE);

			sender.write(filenamebyte,0,filenamebyte.length);

			length = FileTransferBitConverter.getBytes((int)file.length());
			sender.write(length,0,FileTransferBitConverter.INTBITSIZE);

			databyte = new byte[(int)file.length()];
			in.read(databyte,0,databyte.length);
			sender.write(databyte,0,databyte.length);
			if(listener != null){
				listener.progressFileSizeAction(databyte.length,filenamebyte.length);
			}
			if(listener != null){
				listener.uploadComplate();
			}
		}catch(IOException e){
			throw e;
		}finally{
			in.close();
		}
	}
	private void receiveWrite(FileOutputStream out,int length,FileTransferListener listener)
		throws Exception{
		if(isClosed()) throw new SocketException("socket closed");
		if(!isConnected()) throw new SocketException("socket diconnection");

		byte[] buffer = new byte[4096];
		int progressCount = 0;
		while(progressCount < length){
			int bufferSize = 0;
			while((bufferSize = receiver.read(buffer)) > 0){
				out.write(buffer, 0, bufferSize);
				progressCount += bufferSize;
				if(listener != null){
					listener.progressFileSizeAction(progressCount,length);
				}
				if(progressCount >= length){
					break;
				}
			}
		}
	}
	public Exception getLastError(){
		if(errorList.size() > 0){
			Exception e = errorList.pop();
			return e;
		}else{
			return null;
		}
	}
}
